import csv, re, datetime

def ParseDate(tripdate, year):
    if re.match("\d\d\d\d-\d\d-([?][?]|no)", tripdate):
        tripdate = tripdate[:8]+"15"
    mdatestandard = re.search("(\d\d\d\d)-(\d\d)-(\d\d)", tripdate)
    mdategoof = re.match("(\d\d?)/0?(\d)/(20|19)?(\d\d)", tripdate)
    if mdatestandard:
        assert mdatestandard.group(1) == year, (tripdate, year)
        year, month, day = int(mdatestandard.group(1)), int(mdatestandard.group(2)), int(mdatestandard.group(3))
    elif mdategoof:
        assert not mdategoof.group(3) or mdategoof.group(3) == year[:2], mdategoof.groups()
        yadd = int(year[:2]) * 100
        day, month, year = int(mdategoof.group(1)), int(mdategoof.group(2)), int(mdategoof.group(4)) + yadd
    else:
        assert False, tripdate
    return datetime.date(year, month, day)

def ParseTripPersons(trippeople):    
    res = [ ]
    author = None
    for tripperson in re.split(",|\+|&amp;|&(?!\w+;)| and ", trippeople):
        tripperson = tripperson.strip()
        tripperson = re.sub("</?[uU]>", "", tripperson).strip()
        if tripperson and tripperson[0] != '*':
            res.append(tripperson)
    return res


# main parser for pre-2001.  simpler because the data has been hacked so much to fit it
def Parseloghtml01(year, txt, tripdata):
    tripparas = re.findall("<hr\s?/?>([\s\S]*?)(?=<hr\s?/?>)", txt)
    for trippara in tripparas:
        s = re.match(u"(?s)\s*(?:<p>)?(.*?)</?p>(.*)$(?i)", trippara)
        if re.search("omitted\s*write-up|way out man|prick grow|STIFFS RECOVERED", trippara):
            continue
        assert s, trippara[:300]
        tripheader, triptext = s.group(1), s.group(2)
        mtripid = re.search('<a id="(.*?)"', tripheader)
        tripid = mtripid and mtripid.group(1) or ""
        tripheader = re.sub("</?(?:[ab]|span)[^>]*>", "", tripheader)

        #print("ddd   ", [tripheader])
        #continue
        
        tripdate, triptitle, trippeople = tripheader.split("|")
        ldate = ParseDate(tripdate.strip(), year)
    
        mtu = re.search('<p[^>]*>(T/?U.*)', triptext)
        if mtu:
            tu = mtu.group(1)
            triptext = triptext[:mtu.start(0)] + triptext[mtu.end():]
        else:
            tu = ""

        triptitles = triptitle.split(" - ")
        tripcave = triptitles[0].strip()

        ltriptext = triptext
        
        mtail = re.search('(?:<a href="[^"]*">[^<]*</a>|\s|/|-|&amp;|</?p>|\((?:same day|\d+)\))*$', ltriptext)
        if mtail:
            #print mtail.group(0)
            ltriptext = ltriptext[:mtail.start(0)]
        ltriptext = re.sub("</p>", "", ltriptext)
        ltriptext = re.sub("\s*?\n\s*", " ", ltriptext)
        ltriptext = re.sub("<p>|<br>", "\n\n", ltriptext).strip()
        #ltriptext = re.sub("[^\s0-9a-zA-Z\-.,:;'!]", "NONASCII", ltriptext)
        ltriptext = re.sub("</?u>", "_", ltriptext)
        ltriptext = re.sub("</?i>", "''", ltriptext)
        ltriptext = re.sub("</?b>", "'''", ltriptext)
        
        tripdata.append([year, ldate.isoformat(), ParseTripPersons(trippeople), tripcave, triptitle, ltriptext])


def Parseloghtml03(year, txt, tripdata):
    tripparas = re.findall("<hr\s*/>([\s\S]*?)(?=<hr)", txt)
    for trippara in tripparas:
        s = re.match(u"(?s)\s*<p>(.*?)</p>(.*)$", trippara)
        assert s, trippara
        tripheader, triptext = s.group(1), s.group(2)
        tripheader = re.sub("&nbsp;", " ", tripheader)
        tripheader = re.sub("\s+", " ", tripheader).strip()
        sheader = tripheader.split(" -- ")
        tu = ""
        if re.match("T/U|Time underwater", sheader[-1]):
            tu = sheader.pop()
        if len(sheader) != 3:
            print("header not three pieces", sheader)
        tripdate, triptitle, trippeople = sheader
        ldate = ParseDate(tripdate.strip(), year)
        triptitles = triptitle.split(" , ")
        if len(triptitles) >= 2:
            tripcave = triptitles[0]
        else:
            tripcave = "UNKNOWN"
        #print tripcave, "---   ppp", triptitle, trippeople, len(triptext)
        ltriptext = re.sub("</p>", "", triptext)
        ltriptext = re.sub("\s*?\n\s*", " ", ltriptext)
        ltriptext = re.sub("<p>", "\n\n", ltriptext).strip()
        ltriptext = re.sub("[^\s0-9a-zA-Z\-.,:;'!&()\[\]<>?=+*%]", "_NONASCII_", ltriptext)
        tripdata.append([year, ldate.isoformat(), ParseTripPersons(trippeople), tripcave, triptitle, ltriptext])
        
        
def Parseloghtmltxt(year, txt, tripdata):
    txt = re.sub("<!--[\s\S]*?-->", "", txt)
    tripparas = re.findall('(?:<hr\s*/>|(?=<div class="tripdate"))([\s\S]{50}[\s\S]+?)(?=<hr|<div class="tripdate")', txt)
    prevtripdate = ""
    for trippara in tripparas:
        if re.match('\s*<div\sclass="tripdate"\sid="[^"]*?">.*?</div>\s*<p>\s*$', trippara):  # second date
            continue
        s = re.match('''(?x)(?:\s*<div\sclass="tripdate"\sid="[^"]*?">.*?</div>\s*<p>)?  # second date
                            \s*(?:<a\s+id="([^"]*?)"\s*/>\s*</a>)?
                            \s*<div\s+class="tripdate"\s*(?:id="([^"]*?)")?>(.*?)</div>(?:<p>)?
                            \s*<div\s+class="trippeople">\s*(.*?)</div>
                            \s*<div\s+class="triptitle">\s*(.*?)</div>
                            ([\s\S]*?)
                            \s*(?:<div\s+class="timeug">\s*(.*?)</div>)?
                            \s*$
                     ''', trippara)
        if not s:
            if not re.search("(?i)Rigging Guide|rigging topo", trippara):
                print("can't parse: ")
                assert s, [re.sub("<", "&lt;", trippara)]
            continue

        tripid, tripid1, tripdate, trippeople, triptitle, triptext, tu = s.groups()
        if tripdate == "no date":
            tripdate = prevtripdate  # hack in something
        prevtripdate = tripdate
            
        ldate = ParseDate(tripdate.strip(), year)
        #assert tripid[:-1] == "t" + tripdate, (tripid, tripdate)
        trippeople = re.sub("Ol(?!l)", "Olly", trippeople).strip()        
        trippeople = re.sub("Wook(?!e)", "Wookey", trippeople)        
        triptitles = triptitle.split(" - ")
        if len(triptitles) >= 2:
            tripcave = triptitles[0]
        else:
            tripcave = "UNKNOWN"
        #print "\n", tripcave, "---   ppp", trippeople, len(triptext)
        ltriptext = re.sub("</p>", "", triptext)
        ltriptext = re.sub("\s*?\n\s*", " ", ltriptext)
        ltriptext = re.sub("<p>", "\n\n", ltriptext).strip()
        tripdata.append([year, ldate.isoformat(), ParseTripPersons(trippeople), tripcave, triptitle, ltriptext])

        
tripdata = [ ] # [ [year, date, persons, cave, title, text] ]
ybase = "/home/goatchurch/caving/expoweb/years"

for n in range(1982, 2002):
    if n == 1986: continue
    lyear, lloc, parsefunc = "%s"%n, "%d/log.htm"%n, Parseloghtml01
    fin = open(os.path.join(ybase, lloc), encoding="latin1")
    txt = fin.read()
    parsefunc(lyear, txt, tripdata)
    print(lyear, len(tripdata)) #, [k[:3]  for k in tripdata])

for n in range(2002, 2017):
    lyear, lloc, parsefunc = "%s"%n, "%d/logbook.html"%n, (Parseloghtmltxt if n != 2003 else Parseloghtml03)
    fin = open(os.path.join(ybase, lloc), encoding="latin1")
    txt = fin.read()
    parsefunc(lyear, txt, tripdata)
    print(lyear, len(tripdata)) #, [k[:3]  for k in tripdata])

    

import json
k = [ ]
for y in range(1976, 2017):
    ytripdata = [t  for t in tripdata  if int(t[0]) == y]
    june1 = datetime.date(y, 6, 1)
    offsetdays = sorted(list(set((datetime.date(int(t[1][:4]), int(t[1][5:7]), int(t[1][8:10])) - june1).days  for t in ytripdata)))
    ncaverdays = sum(len(t[2])  for t in ytripdata)
    k.append((y, len(ytripdata), ncaverdays, offsetdays))
    
json.dump(k, open("/home/goatchurch/caving/eurospeleotalk/logbookprocessing/logtrips.json", "w"))

